#ifndef lint
static char	*sccsid = "@(#)$Header: misc.c,v 1.41 90/12/12 00:00:02 sob Exp $";
#endif

#include "common.h"
#include "spooldir.h"

extern struct spooldir spdir;
extern struct newscontrol newscont;
extern char group_dir[];


/*
 * open_valid_art -- determine if a given article name is valid;
 *		if it is, return a file pointer to the open article,
 *		along with a unique id of the article.
 *
 *	Parameters:	"artname" is a string containing the
 *			name of the article.
 *			"id" is space for us to put the article
 *			id in.
 *
 *	Returns:	File pointer to the open article if the
 *			article is valid; NULL otherwise
 *
 *	Side effects:	None.
 */

FILE *
open_valid_art(artname, id)
	char		*artname;
	char		*id;
{
	static int	crnt_art_num;
	static char	crnt_art_id[MAXBUFLEN];
	char 		fname[MAXPATHLEN];
	int		fd;
	struct stat	statbuf;

	if (art_fp != NULL) {
		if (crnt_art_num == atoi(artname)) {
			if (fseek(art_fp, (long) 0, 0) < 0)
				close_crnt();
			else {
				(void) strcpy(id, crnt_art_id);
				return (art_fp);
			}
		} else 
			close_crnt();
	}

	(void) sprintf(fname, "%s/%s", group_dir, artname);
	if (spdir.transtype == TYPE_ISO)
	    (void) strcat(fname, iso_extension);
	
	art_fp = fopen(fname, "r");

        /* Need to try to open artname.xrf & read real filename, which
         * will be a full path & name, if the open fails.  Note that this
	 * really only applies for ISO, but it doesn't hurt to try always.
	 */
	if (art_fp == NULL) {
	    char *cp;
	    int templen;
	    /* See if this is a cross referenced article */
	    sprintf(fname, "%s/%s%s", group_dir, artname, XRF_SUFFIX);
#ifdef PATH_DEBUG
printf("No article, trying to open xrf file [%s]\n", fname);
#endif
	    if ((art_fp = fopen(fname, "r")) == NULL)
		return(NULL);
            sprintf(fname, "%s/", spdir.dirpath);
            templen = strlen(fname);
            /* read in the full path for the actual file */
            if (fgets(&fname[templen], sizeof(fname), art_fp) == NULL) {

	        (void) fclose(art_fp);
		return(NULL);
	    }
	    (void) fclose(art_fp);
	    if ((cp = index(fname, '\n')) != NULL)
		*cp = '\0';
            if (spdir.transtype == TYPE_ISO)
                (void) strcat(fname, iso_extension);
#ifdef PATH_DEBUG
printf("Actual article location is [%s]\n", fname);
#endif
	    /* open the actual file */
	    if ((art_fp = fopen(fname, "r")) == NULL)
	        return(NULL);
	}

	fd = fileno(art_fp);

	if (fstat(fd, &statbuf) < 0) {
		close_crnt();
		return (NULL);
	}

	if ((statbuf.st_mode & S_IFMT) != S_IFREG) {
		close_crnt();
		return (NULL);
	}

	get_id(art_fp, id);
	(void) strcpy(crnt_art_id, id);
	crnt_art_num = atoi(artname);
	return (art_fp);
}


/*
 * gethistent -- return the path name of an article if it's
 * in the history file.
 *
 *	Parameters:	"msg_id" is the message ID of the
 *			article, enclosed in <>'s.
 *			"lookup", only check if article exists
 *
 *	Returns:	A char pointer to a static data area
 *			containing the full pathname of the
 *			article, or NULL if the message-id is not
 *			in thef history file.
 *
 *	Side effects:	opens dbm database
 *			(only once, keeps it open after that).
 *			If running Bnews, converts "msg_id" to lower case.
 *			If running Cnews, converts "msg_id" per rfc822.
 *			
 */

#ifndef NDBM
# ifndef DBM
#  ifndef USGHIST
#   define USGHIST
#  endif not USGHIST
# endif not DBM
#endif not DBM

char *
gethistent(msg_id, lookup)
	char		*msg_id;
	int		lookup;
{
	char		line[MAXBUFLEN];
	char		*tmp;
	register char	*cp;
	long		ltmp;
	static char	path[MAXPATHLEN];
#ifdef USGHIST
	char		*histfile();
	register int	len;
#else not USGHIST
#ifdef DBM
	static int	dbopen = 0;
	datum		fetch();
#else not DBM
	static DBM	*db = NULL;	/* History file, dbm version */
#endif DBM
	datum		 key, content;
#endif USGHIST
	static FILE	*hfp = NULL;	/* history file, text version */

#ifdef CNEWS
	cp = rindex(msg_id,'@');	/* look for @ in message id */
	if( cp != NULL)
	{	
	    for(;*cp != '\0';++cp)
#else
	{
	    for (cp = msg_id; *cp != '\0'; ++cp)
#endif
		if (isupper(*cp))
			*cp = tolower(*cp);
/* Make ctags happy */
#ifdef CNEWS
	}
#else
	}
#endif
#ifdef USGHIST
	hfp = fopen(histfile(msg_id), "r");
	if (hfp == NULL) {
		return (NULL);
	}

	len = strlen(msg_id);
	while (fgets(line, sizeof (line), hfp))
		if (!strncasecmp(msg_id, line, len))
			break;

	if (feof(hfp)) {
		(void) fclose(hfp);
		return (NULL);
	}
#else not USGHIST
#ifdef DBM
	if (!dbopen) {
		if (dbminit(newscont.history) < 0) {
			return (NULL);
		} else
			dbopen = 1;
	}
#else	/* ndbm */
	if (db == NULL) {
		db = dbm_open(newscont.history, O_RDONLY, 0);
		if (db == NULL) {
			return (NULL);
		}
	}
#endif DBM

	key.dptr = msg_id;
	key.dsize = strlen(msg_id) + 1;

#ifdef DBM
	content = fetch(key);
#else	/* ndbm */
	content = dbm_fetch(db, key);
#endif DBM
	if (content.dptr == NULL)
		return (NULL);

	/*
	 * If we are just checking to see if it exists return a non-NULL
	 * result
	 */
	if (lookup)
		return ((char *)1);

	if (hfp == NULL) {
		hfp = fopen(newscont.history, "r");
		if (hfp == NULL) {
			return (NULL);
		}
	} else {
/* Why do this if we are going to do an absolute fseek below? XXX */
		rewind(hfp);
	}

	bcopy(content.dptr, (char *)&ltmp, sizeof (long));
	if (fseek(hfp, ltmp, 0) < 0) {
		return (NULL);
	}

	(void) fgets(line, sizeof(line), hfp);
#endif USGHIST

	if ((cp = index(line, '\n')) != NULL)
		*cp = '\0';
	cp = index(line, '\t');
	if (cp != NULL)
		cp = index(cp+1, '\t');
	if (cp == NULL) return(NULL); /* this article has expired */
	tmp = cp+1;

	if ((cp = index(tmp, ' ')) != NULL)
		*cp = '\0';
	
	while ((cp = index(tmp, '.')) != NULL)
		*cp = '/';

	(void) strcpy(path, spdir.dirpath);
	(void) strcat(path, "/");
	(void) strcat(path, tmp);
#ifdef USGHIST
	(void) fclose(hfp);
#endif
	return (path);
}

/*
 * openartbyid -- open an article by message-id.
 *
 *	Arguments:	"msg_id" is the message-id of the article
 *			to open.
 *
 *	Returns:	File pointer to opened article, or NULL if
 *			the article was not in the history file or
 *			could not be opened.
 *
 *	Side effects:	Opens article.
 */

FILE *
openartbyid(msg_id)
char	*msg_id;
{
    FILE	*fptr;
    char	*path;

    path = gethistent(msg_id, 0);
    if (path != NULL) {
        /* Need to try to open <file>.xrf & read real filename, which
         * will be a full path & name, if the open fails.  Note that this
	 * really only applies for ISO, but it doesn't hurt to try always.
	 */
	if ((fptr = fopen(path, "r")) == NULL) {
	    char *cp;
	    int templen;
	    char fname[MAXPATHLEN];
	    /* See if this is a cross referenced article */
	    sprintf(fname, "%s%s", path, XRF_SUFFIX);
#ifdef PATH_DEBUG
printf("No article, trying to open xrf file [%s]\n", fname);
#endif
	    if ((fptr = fopen(fname, "r")) == NULL)
		return(NULL);
            sprintf(fname, "%s/", spdir.dirpath);
            templen = strlen(fname);
            /* read in the full path for the actual file */
            if (fgets(&fname[templen], sizeof(fname), fptr) == NULL) {
	        (void) fclose(fptr);
		return(NULL);
	    }
	    (void) fclose(fptr);
	    if ((cp = index(fname, '\n')) != NULL)
		*cp = '\0';
            if (spdir.transtype == TYPE_ISO)
                (void) strcat(fname, iso_extension);
#ifdef PATH_DEBUG
printf("Actual article location is [%s]\n", fname);
#endif
	    /* open the actual file */
	    if ((fptr = fopen(fname, "r")) == NULL)
	        return(NULL);
	}
	return(fptr);
    } else
	return (NULL);
}


/*
 * check_ngperm -- check to see if they're allowed to see this
 * article by matching Newsgroups: and Distribution: line.
 *
 *	Parameters:	"fp" is the file pointer of this article.
 *
 *	Returns:	0 if they're not allowed to see it.
 *			1 if they are.
 *
 *	Side effects:	None.
 */

check_ngperm(fp)
	register FILE	*fp;
{
	char		buf[MAXBUFLEN];
	register char	*cp;
	static char	**ngarray;
	int		ngcount = 0;

	if (ngpermcount == 0) {
		if (ALLBUT == 0)
			return 0;
		return (1);
	}

	while (fgets(buf, sizeof (buf), fp) != NULL) {
		if (buf[0] == '\n')		/* End of header */
			break;
		if (buf[0] != 'N' && buf[0] != 'n')
			continue;
		cp = index(buf, '\n');
		if (cp)
			*cp = '\0';
		cp = index(buf, ':');
		if (cp == NULL)
			continue;
		*cp = '\0';
		if (!strcasecmp(buf, "newsgroups")) {
			ngcount = get_nglist(&ngarray, cp+2);
			break;
		}
	}

#ifndef USG
	(void) rewind(fp);
#else
	rewind(fp);
#endif
	if (ngcount == 0)	/* Either no newgroups or null entry */
		return (1);

	return (ngmatch(s1strneql, ALLBUT,
		ngpermlist, ngpermcount, ngarray, ngcount));
}


/*
 * get_id -- get the message id of the current article.
 *
 *	Parameters:	"art_fp" is a pointer to the open file.
 *			"id" is space for the message ID.
 *
 *	Returns:	Nothing.
 *
 *	Side effects:	Seeks and rewinds on "art_fp".
 *			Changes space pointed to by "id".
 */

get_id(art_fp, id)
	register FILE	*art_fp;
	char		*id;
{
	char		line[MAXBUFLEN];
	register char	*cp;

	while (fgets(line, sizeof(line), art_fp) != NULL) {
		if (*line == '\n')
			break;
		if (*line == 'M' || *line == 'm') {	/* "Message-ID" */
			if ((cp = index(line, ' ')) != NULL) {
				*cp = '\0';
				if (!strcasecmp(line, "Message-ID:")) {
					(void) strcpy(id, cp + 1);
					if ((cp = index(id, '\n')) != NULL)
						*cp = '\0';
#ifndef USG
					(void) rewind(art_fp);
#else
					rewind(art_fp);
#endif
					return;
				}
			}
		}
	}
#ifndef USG
	(void) rewind(art_fp);
#else
	rewind(art_fp);
#endif
	(void) strcpy(id, "<0>");
}
		

/*
 * close_crnt -- close the current article file pointer, if it's
 *	open.
 *
 *	Parameters:	None.
 *
 *	Returns:	Nothing.
 *
 *	Side effects:	Closes "art_fp" if it's open; sets "art_fp" to NULL.
 */

close_crnt()
{
	if (art_fp != NULL)
		(void) fclose(art_fp);
	art_fp = NULL;
}


/*
 * findart -- find an article number in the article array.
 *
 *	Parameters:	"artname" is a string containing
 *			the name of the article.
 *
 *	Returns:	An index into "art_array",
 *			or -1 if "artname" isn't in "art_array".
 *			
 *	Side effects:	None.
 *
 *	Improvement:	Replace this linear search with a binary one.
 */

findart(artname)
	char		*artname;
{
	register int	i, artnum;

	artnum = atoi(artname);

	for (i = 0; i < num_arts; ++i)
		if (art_array[i] == artnum)
			return(i);

	return (-1);
}


/*
 * get_distlist -- return a nicely set up array of distribution groups
 * along with a count, when given an NNTP-spec distribution list
 * in the form <dist1,dist2,...,distn>.
 *
 *	Parameters:		"array" is storage for our array,
 *				set to point at some static data.
 *				"list" is the NNTP distribution list.
 *
 *	Returns:		Number of distributions found.
 *				-1 on error.
 *
 *	Side effects:		Changes static data area.
 */

get_distlist(array, list)
	char		***array;
	char		*list;
{
	char		*cp;
	int		distcount;
	static char	**dist_list = (char **) NULL;

	if (list[0] != '<')
		return (-1);

	cp = index(list + 1, '>');
	if (cp != NULL)
		*cp = '\0';
	else
		return (-1);

	for (cp = list + 1; *cp != '\0'; ++cp)
		if (*cp == ',')
			*cp = ' ';
	distcount = parsit(list + 1, &dist_list);
	*array = dist_list;
	return (distcount);
}


/*
 * get_nglist -- return a nicely set up array of newsgroups
 * along with a count, when given an NNTP-spec newsgroup list
 * in the form ng1,ng2,ng...
 *
 *	Parameters:		"array" is storage for our array,
 *				set to point at some static data.
 *				"list" is the NNTP newsgroup list.
 *
 *	Returns:		Number of group specs found.
 *
 *	Side effects:		Changes static data area.
 */

get_nglist(array, list)
	char		***array;
	char		*list;
{
	register char	*cp;
	register int	ngcount;

	for (cp = list; *cp != '\0'; ++cp)
		if (*cp == ',')
			*cp = ' ';

	ngcount = parsit(list, array);

	return (ngcount);
}


/*
 * lower -- convert a character to lower case, if it's upper case.
 *
 *	Parameters:	"c" is the character to be
 *			converted.
 *
 *	Returns:	"c" if the character is not
 *			upper case, otherwise the lower
 *			case equivalent of "c".
 *
 *	Side effects:	None.
 */

char
lower(c)
	register char	c;
{
	if (isascii(c) && isupper(c))
		c = c - 'A' + 'a';
	return (c);
}


/* the following is from news 2.11 */

#ifdef USGHIST
/*
** Generate the appropriate history subfile name
*/
char *
histfile(hline)
char *hline;
{
	char chr;	/* least significant digit of article number */
	static char subfile[BUFSIZ];

	chr = findhfdigit(hline);
	sprintf(subfile, "%s.d/%c", HISTORY_FILE, chr);
	return subfile;
}

findhfdigit(fn)
char *fn;
{
	register char *p;
	register int chr;

	p = index(fn, '@');
	if (p != NULL && p > fn)
		chr = *(p - 1);
	else
		chr = '0';
	if (!isdigit(chr))
		chr = '0';
	return chr;
}
#endif USGHIST
#ifdef USG
#ifndef GAZETTE
bcopy(s, d, l)
	register char *s, *d;
	register int l;
{
	while (l-- > 0)
		*d++ = *s++;
}
#endif !GAZETTE
#endif USG

/*
 * determine_iso_extension tries to determine the iso extension that the
 * cdrom driver slaps on the end of filenames which have no extension.
 * Files which fit this category are "active", "pathtran", <articles>, ...
 * Various drivers may put a dot "." or ".;1" on the end of the filenames.
 * This is just an on-the-fly way of working around this problem.
 */

int determine_iso_extension(fname, extension)
char *fname;
char *extension;
{
    char *iso_ext, *getenv();
    FILE *tmpptr;
    int  orig_len;

    if ((tmpptr = fopen(fname, "r")) == NULL) {
        iso_ext = getenv("ISO_EXTENSION");
        if (iso_ext != NULL) {
	    orig_len = strlen(fname);
	    strcat(fname, iso_ext);
            if ((tmpptr = fopen(fname, "r")) != NULL) {
		strcpy(extension, iso_ext);
		(void) fclose(tmpptr);
		return(1);
	    }
	    fname[orig_len] = '\0';
	}
	strcat(fname, ".");
        if ((tmpptr = fopen(fname, "r")) == NULL) {
	    strcat(fname, ";1");
            if ((tmpptr = fopen(fname, "r")) == NULL)
		return(0); /* failed to determine the extension */
	    else /* extension is ".;1" */
	        strcpy(iso_extension, ".;1");
	} else /* extension is "." */
	    strcpy(iso_extension, ".");
    } else
	iso_extension[0] = '\0';
    (void) fclose(tmpptr);
    return(1);
}
