/*
** @(#)spooldir.c	1.2 5/15/92
**
** This function is used to return a spool directory 
** record for the named spool directory.
**/
#include <stdio.h>
#include "conf.h"
#include "spooldir.h"

char *fgets();
FILE *fopen();

static FILE *spf = NULL;
static struct spooldir s;

void setspoolent()
{
    char *ptr;

    if (spf == NULL) {
	/*
	 * First check if the user has set shell variable XSPOOLDIR
	 * and then try the default SPOOLDIRS_FILE
	 */
	ptr = (char *) getenv ("XSPOOLDIR");
	if (ptr != (char *) 0) {
            spf = fopen(ptr, "r");
	} else {
            spf = fopen(SPOOLDIRS_FILE, "r");
	}
    } else
        rewind(spf);
}

void endspoolent()
{
    if(spf != NULL) {
        (void) fclose(spf);
        spf = NULL;
    }
}

int getspooldir(spname, sdir)
    char *spname;
    struct spooldir *sdir;
{
    char isobuf[BUFSIZ];
    char trans[BUFSIZ];
    int rc;
    struct spooldir p;

    /*
    ** lookup time...
    */

    setspoolent();

    if (spf == NULL)    /* did the open fail ? */
        return(-2);
    
    rc = 0;

    while (fgets(isobuf, sizeof(isobuf), spf) != NULL) {
	if (*isobuf == '#')     /* skip comments */
            continue;

        /*
        ** If the line does not have five entries on it
        ** then silently ignore it.
        */

        if (sscanf(isobuf,"%s %s %s %s %s", p.name,p.mntpoint,p.libpath,
			p.dirpath,trans) != 5)
            continue;

        if (strcmp(spname,p.name) == 0) {
            if (strcasecmp(trans,"ISO") == 0) 
                p.transtype = TYPE_ISO;
            else
                p.transtype = TYPE_UNIX;
            *sdir = p;
            if (is_available(sdir))
                rc = 1;
            else
                rc = -1;
        }

    }
    endspoolent();
    return(rc);
}

struct spooldir *getspoolent(spldir_sts)
int *spldir_sts; /* 1 if available, 0 if not available */
{
    int line_ok;
    char ibuf[BUFSIZ+1];
    char trans[80];

    if (spf == NULL) {
        if((spf = fopen(SPOOLDIRS_FILE, "r")) == NULL)
            return(NULL);
    }

    line_ok = 0;

    do {
        if (fgets(ibuf, sizeof(ibuf), spf) == NULL) 
            return(NULL);

        if (*ibuf != '#') {    /* skip comments */
	    
            /*
            ** If the line does not have five entries on it
            ** then silently ignore it.
            */
            if (sscanf(ibuf,"%s %s %s %s %s %80[^\n\r]", s.name, s.mntpoint,
                       s.libpath,s.dirpath,trans,s.descrip) >= 5) {
                if (is_available(&s))
                    *spldir_sts = 1; /* available */
		else
                    *spldir_sts = 0; /* not available */
		line_ok = 1;
	    }
        }
    } while (!line_ok); 

    if (strcasecmp(trans,"ISO") == 0) 
        s.transtype = TYPE_ISO;
    else
        s.transtype = TYPE_UNIX;
    return(&s); 
}


char *vol_issue[] = { 
".cddate",	/* Original Sun UFS CD-ROM */
"issue.vol",	/* ISO and Disk Archives   */	
 };

int num_files = (sizeof (vol_issue)/ sizeof(char *));

int is_available(sdir)
    struct spooldir *sdir;
{
    char *strchr();

    char *rp;
    register int i;
    char dtstr[256];
    char dtfile[1024];
    FILE *fp;

    /*
    ** Check to see if the spooldir in "news".  If it is check 
    ** for the existance of the active file in the lib directory.
    ** Otherwise check the date in the issue file in the archive's
    ** upper level directory...
    */

    if (strcmp("news", sdir->name) == 0) {
        (void) sprintf(dtfile,"%s/%s", sdir->libpath,ACTIVE_FILE);
        if ((fp = fopen(dtfile,"r")) != NULL) {
             /* assume its available */
             (void) fclose(fp);
             return(1);
        }
        return(0);
    }

    /*  check the top level archive directory for a file listed in vol_issue */
        
    for (i = 0; i < num_files; i++) {
        (void) sprintf(dtfile,"%s/%s", sdir->mntpoint,vol_issue[i]);
        if ((fp = fopen(dtfile,"r")) != NULL) {
            while (fgets(dtstr, sizeof(dtstr), fp) != NULL) {
	        if (*dtstr == '#')     /* skip comments */
                    continue;
                if ((rp = strchr(dtstr,'\n')) != NULL)
                    *rp = '\0';
            }
 
            if (strcmp(dtstr, sdir->name) == 0) {
                (void) fclose(fp);
                return(1);
            }
            (void) fclose(fp);
        }
    }
    return(0);
}
