#ifndef lint
static char	*sccsid = "@(#)$Header: xhdr.c,v 1.11 90/12/11 23:59:53 sob Exp $";
#endif

#include "common.h"
#include "spooldir.h"

#ifdef XHDR

extern struct spooldir spdir;
extern char group_dir[];

int	artptr;
int	artnum;
FILE	*fp1;
int	low, high;
char    header_field[50];
char    msg_id[200];
/*
 * XHDR header [<messageid>|articlerange]
 *
 * header is a case-insensitive header field, minus any colons.
 *
 * articlerange is one of:
 *	an article number
 *	an article number followed by a dash to indicate all following
 *	an article number followed by a dash followed by another
 *		article number.
 * e.g.,
 * XHDR subject			retrieve subject of current article
 * XHDR subject 5589-6325	retrieve subject of arts 5589 to 6325
 * XHDR subject 5589-		retrieve subject of arts 5589 and up
 * XHDR subject 5589		retrieve subject of art 5589 only
 * XHDR subject <123@ucbvax>	retrieve subject of art <123@ucbvax>
 *
 * This command is an extension, and not included in RFC 977.
 */

xhdr(argc, argv)
int	argc;
char	*argv[];
{
    register char	*cp;

    if (argc < 2 || argc > 3) {
        state_cb.state = SINGLE_RESP_STATE;
	(void) sprintf(response_text, 
                "%d Usage: XHDR headerfield [artrange|<message-id>]\r\n",
		ERR_CMDSYN);
	return;
    }

    /* Handle message-id requests */

    if (argc == 3 && *argv[2] == '<') {	/* Message ID */
	fp1 = openartbyid(argv[2]);
	if (fp1 == NULL) {
            state_cb.state = SINGLE_RESP_STATE;
	    (void) sprintf(response_text, 
                           "%d No article by message-id %s, sorry.\r\n",
			ERR_NOART, argv[2]);
	    return;
	}
        state_cb.state = XHDR_MSG_ID_STATE;
	(void) sprintf(response_text, "%d 0 %s header of article %s.\r\n",
			OK_HEAD, argv[1], argv[2]);
        (void) strncpy(header_field, argv[1], 50);
        header_field[49] = '\0';
        (void) strncpy(msg_id, argv[2], 200);
        msg_id[199] = '\0';
        return;
    }

    /*
     * It must be a range of articles, which means that we need
     * to be in a newsgroup already.
     */

    if (!ingroup) {
        state_cb.state = SINGLE_RESP_STATE;
	(void) sprintf(response_text, 
                "%d You are not currently in a newsgroup.\r\n",
		ERR_NCING);
	return;
    }

    artptr = 0;

    if (argc == 2) {
	if (art_ptr < 0 || art_ptr >= num_arts) {
            state_cb.state = SINGLE_RESP_STATE;
            (void) sprintf(response_text, 
                       "%d No article is currently selected.\r\n",
			ERR_NOCRNT);
	    return;
	}
	high = low = art_array[art_ptr];
	artptr = art_ptr;
    } else {
	cp = index(argv[2], '-');
	if (cp == NULL)
	    low = high = atoi(argv[2]);
	else {
	    *cp = '\0';
	    low = atoi(argv[2]);
	    cp++;
	    high = atoi(cp);
	    if (high < low)
		high = art_array[num_arts-1];
	}
    }

    state_cb.state = XHDR_STATE;
    (void) sprintf(response_text, "%d %s fields follow\r\n", OK_HEAD, argv[1]);
    (void) strncpy(header_field, argv[1], 50); /* save argv[1] for xhdr_read */
    header_field[49] = '\0';
}


xhdr_msg_id_read()
{
    static char first_time = 1;

    if (first_time) {
        (void) print_header(fp1, header_field, msg_id);
	(void) fclose(fp1);
        first_time = 0;
    } else {
        (void) strcpy(response_text, ".\r\n");
        first_time = 1;
    }
    return;
}


xhdr_read()
{
    char		buf[MAXPATHLEN];
    char		fname[MAXPATHLEN];
    char		artname[10];
    register FILE	*fp;

    for (;artptr<num_arts; artptr++) {
	if ((artnum = art_array[artptr]) < low)
	    continue;
	if (artptr >= num_arts || artnum > high)
	    break;

        (void) sprintf(buf, "%s/%d", group_dir, artnum);
        if (spdir.transtype == TYPE_ISO)
            (void) strcat(buf, iso_extension);

	fp = fopen(buf, "r");

        /* Need to try to open artnum.xrf & read real filename, which
         * will be a full path & name, if the open fails.  Note that this
         * really only applies for ISO, but it doesn't hurt to try always.
         */
	if (fp == NULL) {
            char *cp;
            int templen;
            /* See if this is a cross referenced article */
            (void) sprintf(buf, "%s/%d%s", group_dir, artnum, XRF_SUFFIX);
#ifdef PATH_DEBUG
printf("No article, trying to open xrf file [%s]\n", fname);
#endif
            if ((fp = fopen(buf, "r")) == NULL)
                continue;
            sprintf(fname, "%s/", spdir.dirpath);
            templen = strlen(fname);
            /* read in the full path for the actual file */
            if (fgets(&fname[templen], sizeof(fname), fp) == NULL) {

                (void) fclose(fp);
                continue;
            }
            (void) fclose(fp);
            if ((cp = index(fname, '\n')) != NULL)
                *cp = '\0';
            if (spdir.transtype == TYPE_ISO)
                (void) strcat(fname, iso_extension);
#ifdef PATH_DEBUG
printf("Actual article location is [%s]\n", fname);
#endif
            /* open the actual file */
            if ((fp = fopen(fname, "r")) == NULL)
	        continue;
	}
	sprintf(artname, "%d", artnum);
	(void) print_header(fp, header_field, artname);
	(void) fclose(fp);
        artptr++; /* need this because of return */
        return; /* let a call to get_server get this line */
    }
    state_cb.state = NULL_STATE;
    (void) strcpy(response_text, ".\r\n");
    return;
}


static int print_header(fp, header, artname)
register FILE	*fp;
register char	*header;
register char	*artname;
{
    char		line[NNTP_STRLEN];
    register char	*cp, *cp1;

    while (fgets(line, sizeof (line), fp) != NULL) {
	if (*line == '\n' || *line == '\0') {
	    (void) sprintf(response_text, "%s (none)\r\n", artname);
	    return;
	}
	if (cp = index(line, ':')) {
	    *cp = '\0';
	    if (!strcasecmp(header, line)) {
		if (cp1 = index(cp + 2, '\n'))
		    *cp1 = '\0';
		(void) sprintf(response_text, "%s %s\r\n", artname, cp + 2);
		return;
	    }
	}
    }
}

#else /* not XHDR */

/* Kludge to get around Greenhills C compiler */

xhdr_greenkluydge()
{
}

#endif /* not XHDR */
